/* 
        HASH.C
         Created           : 15-SEP-1992 by Christian Arnault
*/
 
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include <Hash.h>

static char Alpha[] = "abcdefghijklmnopqrstuvwxyz";

int irandom (int range);

/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Static definitions.                                                */
/*                                                                     */
/*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
typedef char* CiSymbol;

#ifdef __cplusplus
extern "C" {
#endif

CiSymbol CiSymbolNew (char* name);
void CiSymbolDelete (CiSymbol This);
CiSymbol CiSymbolGetReference (char* name);

#ifdef __cplusplus
};
#endif
/*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
static HTable HashTable = 0;
/*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
main (int argc, char** argv)
{
  int tryNum;
  int number;
  int pos;
  char text[33];
  int tries = 1000;
  int size = 1000;

  if (argc >= 2) sscanf (argv[1], "%d", &tries);
  if (argc == 3) sscanf (argv[2], "%d", &size);

  HashTable = HTableNew (size, 0);
  
  printf ("---> Entering symbols\n");
  
#ifdef vms
  lib$init_timer();
#endif

  for (tryNum = 0; tryNum < tries; tryNum++)
  {
    number = irandom (32);
    for (pos = 0; pos < number; pos++)
    {
      text[pos] = Alpha[irandom(sizeof(Alpha) - 1)];
    }
    text[number] = 0;
  
    CiSymbolNew (text);
  }

#ifdef vms
  lib$show_timer();
#endif

  printf ("---> Accessing symbols by hash\n");
  
#ifdef vms
  lib$init_timer();
#endif

  for (tryNum = 0; tryNum < tries; tryNum++)
  {
    number = irandom (32);
    for (pos = 0; pos < number; pos++)
    {
      text[pos] = Alpha[irandom(sizeof(Alpha) - 1)];
    }
    text[number] = 0;
  
    CiSymbolGetReference (text);
  }

#ifdef vms
  lib$show_timer();
#endif
  exit( 0 );
}

/*- procedure ---------------------------------------------------------*/
int irandom (int range)
/*---------------------------------------------------------------------*/
{
  double x;
  
  x = ((double) rand()) / ((double) 0x7fffffff);
  x *= range;
  return ((int) x);
}
 
/*---------------------------------------------------------------------*/
/*                                                                     */
/*  Symbol    management.                                              */
/*                                                                     */
/*---------------------------------------------------------------------*/

/*- procedure ---------------------------------------------------------*/
CiSymbol CiSymbolNew (char* name)
/*---------------------------------------------------------------------*/
{
  CiSymbol This;

  This = (CiSymbol) HTableFindReference (HashTable, name);
  if (This) return (This);

  This = (CiSymbol) malloc (strlen(name) + 1);
  strcpy (This, name);

  HTableAddReference (HashTable, (HReference) This);

  return (This);
}


/*- procedure ---------------------------------------------------------*/
void CiSymbolDelete (CiSymbol This)
/*---------------------------------------------------------------------*/
{
  if (!This) return;
  
  HTableRemoveReference (HashTable, (HReference) This);

  free (This);
}

/*- procedure ---------------------------------------------------------*/
CiSymbol CiSymbolGetReference (char* name)
/*---------------------------------------------------------------------*/
{
  return (HTableFindReference (HashTable, name));
}

